@@ -4,7 +4,7 @@ import numbers
 import operator as op
 import re
 
-from functions import first, identity, is_seq, last, merge, walk
+from functions import assoc, first, identity, is_seq, last, walk
 
 
 eq = lambda val: partial(op.eq, val)
@@ -106,8 +106,18 @@ def validate_with(schema):
     """Validate function arguments and check required fields."""
     def decorator(f):
         @wraps(f)
-        def wrapper(data, *args, **kwargs):
-            return f(validate(data, schema), *args, **kwargs)
+        def wrapper(*args, **kwargs):
+            if not kwargs['query_params']:
+                query_params = {}
+            else:
+                query_params = validate(kwargs['query_params'], schema)
+            if not kwargs['form_params']:
+                form_params = {}
+            else:
+                form_params = validate(kwargs['form_params'], schema)
+                kwargs.update({'query_params': query_params,
+                               'form_params': form_params})
+            return f(**kwargs)
         return wrapper
     return decorator
 
@@ -127,7 +137,7 @@ def marshal(data, schema, before=False):
                 "Cannot process node for key '{0}' and value '{1}'".format(k, v1))
 
     def handler(k):
-        print "Cannot process '{0}', key not in schema".format(k)
+#        print "Cannot process '{0}', key not in schema".format(k)
         return None
     return walk_pair(partial(marshal_node, before=before), identity, data, schema,
                      handler)
@@ -137,28 +147,20 @@ def marshal_with(schema):
     def decorator(f):
         @wraps(f)
         def wrapper(*args, **kwargs):
-            if args:
-                data = marshal(first(args), schema, before=True)
-                rmap = f(data, **kwargs)
-                body = marshal(rmap['body'], schema)
-                return merge(rmap, {'body': body})
+            if not kwargs['query_params']:
+                query_params = {}
             else:
-                rmap = f(*args, **kwargs)
-                body = marshal(rmap['body'], schema)
-                return merge(rmap, {'body': body})
+                query_params = marshal(kwargs['query_params'], schema,
+                                       before=True)
+            if not kwargs['form_params']:
+                form_params = {}
+            else:
+                form_params = marshal(kwargs['form_params'], schema,
+                                      before=True)
+            kwargs.update({'query_params': query_params,
+                           'form_params': form_params})
+            response_map = f(**kwargs)
+            body = marshal(response_map['body'], schema)
+            return assoc(response_map, 'body', body)
         return wrapper
     return decorator
-
-
-# this doesn't work'
-# def satisfies(schema, data):
-#     def assert_at_node(k, validator, v):
-#         if validator.__class__ == partial and validator.func == op.eq:
-#             assert v == first(validator.args)
-#         else:
-#             assert validator(v)
-#         return None
-
-#     def handler(k):
-#         raise AssertionError("Cannot validate '{0}', key not in data".format(k))
-#     return walk_pair(assert_at_node, identity, data, sanitize_keys(schema), handler)
